@@ -5,14 +5,17 @@ using swoole::Timer;
 using swoole::coroutine::System;
 using swoole::test::coroutine;
 
+const char *domain_baidu = "www.baidu.com";
+const char *domain_tencent = "www.tencent.com";
+
 TEST(coroutine_gethostbyname, resolve_cache) {
     coroutine::run([](void *arg) {
         System::set_dns_cache_capacity(10);
-        std::string addr1 = System::gethostbyname("www.baidu.com", AF_INET);
+        std::string addr1 = System::gethostbyname(domain_baidu, AF_INET);
         ASSERT_NE(addr1, "");
         int64_t with_cache = Timer::get_absolute_msec();
         for (int i = 0; i < 100; ++i) {
-            std::string addr2 = System::gethostbyname("www.baidu.com", AF_INET);
+            std::string addr2 = System::gethostbyname(domain_baidu, AF_INET);
             ASSERT_EQ(addr1, addr2);
         }
         with_cache = Timer::get_absolute_msec() - with_cache;
@@ -20,7 +23,7 @@ TEST(coroutine_gethostbyname, resolve_cache) {
         System::set_dns_cache_capacity(0);
         int64_t without_cache = Timer::get_absolute_msec();
         for (int i = 0; i < 5; ++i) {
-            std::string addr2 = System::gethostbyname("www.baidu.com", AF_INET);
+            std::string addr2 = System::gethostbyname(domain_baidu, AF_INET);
             ASSERT_NE(addr2, "");
         }
         without_cache = Timer::get_absolute_msec() - without_cache;
@@ -31,7 +34,7 @@ TEST(coroutine_gethostbyname, resolve_cache) {
 
 TEST(coroutine_gethostbyname, impl_async) {
     coroutine::run([](void *arg) {
-        auto result = swoole::coroutine::gethostbyname_impl_with_async("www.baidu.com", AF_INET);
+        auto result = swoole::coroutine::gethostbyname_impl_with_async(domain_baidu, AF_INET);
         ASSERT_EQ(result.empty(), false);
     });
 }
@@ -61,3 +64,23 @@ TEST(coroutine_gethostbyname, resolve_cache_inet4_and_inet6) {
         ASSERT_LT(Timer::get_absolute_msec() - start, 5);
     });
 }
+
+TEST(coroutine_gethostbyname, dns_expire) {
+    coroutine::run([](void *arg) {
+        time_t expire = 0.2;
+        System::set_dns_cache_expire(expire);
+        System::gethostbyname(domain_tencent, AF_INET);
+
+        int64_t with_cache = Timer::get_absolute_msec();
+        System::gethostbyname(domain_tencent, AF_INET);
+        with_cache = Timer::get_absolute_msec() - with_cache;
+
+        sleep(0.3);
+        int64_t without_cache = Timer::get_absolute_msec();
+        System::gethostbyname(domain_tencent, AF_INET);
+        without_cache = Timer::get_absolute_msec() - without_cache;
+
+        ASSERT_GE(without_cache, with_cache);
+        System::clear_dns_cache();
+    });
+}
