@@ -1,5 +1,5 @@
 <?php
-$serv = new swoole_server("0.0.0.0", 9501);
+$serv = new Swoole\Server("0.0.0.0", 9501);
 $serv->on('connect', function ($serv, $fd, $reactor_id){
 	echo "[#".posix_getpid()."]\tClient@[$fd]: Connect.\n";
 });
@@ -8,7 +8,7 @@ $serv->set(array(
 
 ));
 
-$serv->on('receive', function (swoole_server $serv, $fd, $reactor_id, $data) {
+$serv->on('receive', function (Swoole\Server $serv, $fd, $reactor_id, $data) {
 	echo "[#".$serv->worker_id."]\tClient[$fd] receive data: $data\n";
 	if ($serv->send($fd, "{$data}\n") == false)
 	{
