@@ -7,7 +7,7 @@
  * SWOOLE_RWLOCK 读写锁
  */
 
-$lock = new swoole_lock(SWOOLE_MUTEX);
+$lock = new Swoole\Lock(SWOOLE_MUTEX);
 echo "[Master]create lock\n";
 $lock->lock();
 if (pcntl_fork() > 0)
@@ -27,4 +27,3 @@ echo "[Master]release lock\n";
 unset($lock);
 sleep(1);
 echo "[Master]exit\n";
-
