@@ -21,4 +21,4 @@ go(function () {
     }
 });
 
-swoole_event_wait();
+Swoole\Event::wait();
