@@ -1,5 +1,5 @@
 <?php
-$serv = new swoole_server("0.0.0.0", 9501);
+$serv = new Swoole\Server("0.0.0.0", 9501);
 //$serv->on('connect', function ($serv, $fd, $reactor_id){
 //	echo "[#".posix_getpid()."]\tClient@[$fd:$reactor_id]: Connect.\n";
 //});
@@ -7,7 +7,7 @@ $serv->set(array(
     'worker_num' => 1,
 ));
 
-$serv->on('receive', function (swoole_server $serv, $fd, $reactor_id, $data) {
+$serv->on('receive', function (Swoole\Server $serv, $fd, $reactor_id, $data) {
     usleep(rand(100000, 2000000));
     var_dump(round($serv->getReceivedTime(), 10));
 });
@@ -17,4 +17,3 @@ $serv->on('receive', function (swoole_server $serv, $fd, $reactor_id, $data) {
 //});
 
 $serv->start();
-
