@@ -1,5 +1,5 @@
 <?php
-$client = new swoole_client(SWOOLE_SOCK_TCP);
+$client = new Swoole\Client(SWOOLE_SOCK_TCP);
 
 if (!$client->connect('127.0.0.1', 9504)) {
     exit("connect failed\n");
