@@ -1,6 +1,6 @@
 <?php
-$serv = new swoole_server("0.0.0.0", 9501, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
-// $serv = new swoole_server("0.0.0.0", 9501, SWOOLE_PROCESS, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+$serv = new Swoole\Server("0.0.0.0", 9501, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+// $serv = new Swoole\Server("0.0.0.0", 9501, SWOOLE_PROCESS, SWOOLE_SOCK_TCP | SWOOLE_SSL);
 $key_dir = dirname(dirname(__DIR__)).'/tests/ssl';
 
 // $port2 = $serv->addlistener('0.0.0.0', 9502, SWOOLE_SOCK_TCP);
@@ -18,13 +18,13 @@ $serv->set(array(
     'ssl_verify_depth' => 10,
 ));
 
-$serv->on('connect', function (swoole_server $serv, $fd, $reactor_id){
+$serv->on('connect', function (Swoole\Server $serv, $fd, $reactor_id){
 	echo "[#".posix_getpid()."]\tClient@[$fd:$reactor_id]: Connect.\n";
     $info = $serv->getClientInfo($fd);
     var_dump($info);
 });
 
-$serv->on('receive', function (swoole_server $serv, $fd, $reactor_id, $data) {
+$serv->on('receive', function (Swoole\Server $serv, $fd, $reactor_id, $data) {
 	echo "[#".posix_getpid()."]\tClient[$fd]: $data\n";
 	$serv->send($fd, "Swoole: $data\n");
 });
