@@ -1,7 +1,7 @@
 <?php
 $ssl_dir = realpath('../../tests/ssl');
-$serv = new swoole_websocket_server("0.0.0.0", 9502, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
-//$serv = new swoole_websocket_server("0.0.0.0", 9502, SWOOLE_PROCESS, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+$serv = new Swoole\WebSocket\Server("0.0.0.0", 9502, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+//$serv = new Swoole\WebSocket\Server("0.0.0.0", 9502, SWOOLE_PROCESS, SWOOLE_SOCK_TCP | SWOOLE_SSL);
 $serv->set([
     'ssl_cert_file' => $ssl_dir . '/ssl.crt',
     'ssl_key_file' => $ssl_dir . '/ssl.key',
@@ -18,7 +18,7 @@ $serv->on('connect', function ($_server, $fd) {
     echo "client {$fd} connect\n";
 });
 
-$serv->on('open', function (swoole_websocket_server $_server, swoole_http_request $request) {
+$serv->on('open', function (Swoole\WebSocket\Server $_server, Swoole\Http\Request $request) {
     echo "server#{$_server->worker_pid}: handshake success with fd#{$request->fd}\n";
 //    var_dump($request);
 });
@@ -27,7 +27,7 @@ $serv->on('request', function ($req, $resp) {
     $resp->end(file_get_contents(__DIR__.'/websocket_client.html'));
 });
 
-$serv->on('message', function (swoole_websocket_server $_server, $frame) {
+$serv->on('message', function (Swoole\WebSocket\Server $_server, $frame) {
     var_dump($frame->data);
     echo "received ".strlen($frame->data)." bytes\n";
     $_send = str_repeat('B', rand(100, 800));
