@@ -1,5 +1,5 @@
 <?php
-$serv = new swoole_server(__DIR__."/svr.sock", 9501, SWOOLE_BASE, SWOOLE_SOCK_UNIX_STREAM);
+$serv = new Swoole\Server(__DIR__."/svr.sock", 9501, SWOOLE_BASE, SWOOLE_SOCK_UNIX_STREAM);
 $serv->set(array(
     //'tcp_defer_accept' => 5,
     'worker_num' => 1,
@@ -19,10 +19,10 @@ $serv->on('Close', function($serv, $fd, $reactorId) {
     echo "Close, client={$fd}\n";
 });
 
-$serv->on('receive', function (swoole_server $serv, $fd, $from_id, $data)
+$serv->on('receive', function (Swoole\Server $serv, $fd, $reactor_id, $data)
 {
     echo "[#" . posix_getpid() . "]\tClient[$fd]: $data\n";
-    $serv->send($fd, json_encode(array("hello" => $data, "from" => $from_id)) . PHP_EOL);
+    $serv->send($fd, json_encode(array("hello" => $data, "from" => $reactor_id)) . PHP_EOL);
 });
 
 $serv->start();
