@@ -32,6 +32,7 @@ END_EXTERN_C()
 
 using namespace swoole;
 using swoole::coroutine::Socket;
+using swoole::http2::get_default_setting;
 
 namespace Http2 = swoole::http2;
 
@@ -74,7 +75,8 @@ class Client {
     bool open_ssl;
     double timeout = network::Socket::default_read_timeout;
 
-    Socket *client = nullptr;
+    Socket *socket_ = nullptr;
+    zval socket_object;
 
     nghttp2_hd_inflater *inflater = nullptr;
     nghttp2_hd_deflater *deflater = nullptr;
@@ -85,6 +87,10 @@ class Client {
     Http2::Settings local_settings = {};
     Http2::Settings remote_settings = {};
 
+    // flow control
+    uint32_t remote_window_size = 0;
+    uint32_t local_window_size = 0;
+
     std::unordered_map<uint32_t, Stream *> streams;
     std::queue<zend_string *> send_queue;
 
@@ -99,6 +105,7 @@ class Client {
         _zobject = *__zobject;
         zobject = &_zobject;
         Http2::init_settings(&local_settings);
+        local_window_size = local_settings.init_window_size;
     }
 
     inline Stream *get_stream(uint32_t stream_id) {
@@ -113,12 +120,11 @@ class Client {
     ssize_t build_header(zval *zobject, zval *zrequest, char *buffer);
 
     inline void update_error_properties(int code, const char *msg) {
-        zend_update_property_long(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errCode"), code);
-        zend_update_property_string(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errMsg"), msg);
+        php_swoole_socket_set_error_properties(zobject, code, msg);
     }
 
     inline void io_error() {
-        update_error_properties(client->errCode, client->errMsg);
+        update_error_properties(socket_->errCode, socket_->errMsg);
     }
 
     inline void nghttp2_error(int code, const char *msg) {
@@ -126,7 +132,7 @@ class Client {
     }
 
     inline bool is_available() {
-        if (sw_unlikely(!client || !client->is_connected())) {
+        if (sw_unlikely(!socket_ || !socket_->is_connected())) {
             swoole_set_last_error(SW_ERROR_CLIENT_NO_CONNECTION);
             zend_update_property_long(
                 swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errCode"), SW_ERROR_CLIENT_NO_CONNECTION);
@@ -140,13 +146,13 @@ class Client {
     }
 
     inline void apply_setting(zval *zset) {
-        if (client && ZVAL_IS_ARRAY(zset)) {
-            php_swoole_client_set(client, zset);
+        if (socket_ && ZVAL_IS_ARRAY(zset)) {
+            php_swoole_socket_set(socket_, zset);
         }
     }
 
     inline bool recv_packet(double timeout) {
-        if (sw_unlikely(client->recv_packet(timeout) <= 0)) {
+        if (sw_unlikely(socket_->recv_packet(timeout) <= 0)) {
             io_error();
             return false;
         }
@@ -177,6 +183,7 @@ class Client {
     bool send_goaway_frame(zend_long error_code, const char *debug_data, size_t debug_data_len);
     ReturnCode parse_frame(zval *return_value, bool pipeline_read = false);
     bool close();
+    void socket_dtor();
 
     ~Client() {
         close();
@@ -195,23 +202,23 @@ class Client {
     }
 
     inline bool send(const char *buf, size_t len) {
-        if (client->has_bound(SW_EVENT_WRITE)) {
+        if (socket_->has_bound(SW_EVENT_WRITE)) {
             if (send_queue.size() > remote_settings.max_concurrent_streams) {
-                client->errCode = SW_ERROR_QUEUE_FULL;
-                client->errMsg = "the send queue is full, try again later";
+                socket_->errCode = SW_ERROR_QUEUE_FULL;
+                socket_->errMsg = "the send queue is full, try again later";
                 io_error();
                 return false;
             }
             send_queue.push(zend_string_init(buf, len, 0));
             return true;
         }
-        if (sw_unlikely(client->send_all(buf, len) != (ssize_t) len)) {
+        if (sw_unlikely(socket_->send_all(buf, len) != (ssize_t) len)) {
             io_error();
             return false;
         }
         while (send_queue.size() > 0) {
             zend_string *frame = send_queue.front();
-            if (sw_unlikely(client->send_all(frame->val, frame->len) != (ssize_t) frame->len)) {
+            if (sw_unlikely(socket_->send_all(frame->val, frame->len) != (ssize_t) frame->len)) {
                 io_error();
                 zend_throw_exception(swoole_http2_client_coro_exception_ce,
                                      "failed to send control frame",
@@ -234,25 +241,21 @@ using swoole::coroutine::http2::Stream;
 using swoole::http2::HeaderSet;
 
 struct Http2ClientObject {
-    Client *h2c;
+    Client *client;
     zend_object std;
 };
 
-static sw_inline Http2ClientObject *php_swoole_http2_client_coro_fetch_object(zend_object *obj) {
+static sw_inline Http2ClientObject *http2_client_coro_fetch_object(zend_object *obj) {
     return (Http2ClientObject *) ((char *) obj - swoole_http2_client_coro_handlers.offset);
 }
 
-static sw_inline Client *php_swoole_get_h2c(zval *zobject) {
-    return php_swoole_http2_client_coro_fetch_object(Z_OBJ_P(zobject))->h2c;
+static sw_inline Client *http2_client_coro_get_client(zval *zobject) {
+    return http2_client_coro_fetch_object(Z_OBJ_P(zobject))->client;
 }
 
-static sw_inline void php_swoole_set_h2c(zval *zobject, Client *h2c) {
-    php_swoole_http2_client_coro_fetch_object(Z_OBJ_P(zobject))->h2c = h2c;
-}
-
-static void php_swoole_http2_client_coro_free_object(zend_object *object) {
-    Http2ClientObject *request = php_swoole_http2_client_coro_fetch_object(object);
-    Client *h2c = request->h2c;
+static void http2_client_coro_free_object(zend_object *object) {
+    Http2ClientObject *request = http2_client_coro_fetch_object(object);
+    Client *h2c = request->client;
 
     if (h2c) {
         delete h2c;
@@ -260,7 +263,7 @@ static void php_swoole_http2_client_coro_free_object(zend_object *object) {
     zend_object_std_dtor(&request->std);
 }
 
-static zend_object *php_swoole_http2_client_coro_create_object(zend_class_entry *ce) {
+static zend_object *http2_client_coro_create_object(zend_class_entry *ce) {
     Http2ClientObject *request = (Http2ClientObject *) zend_object_alloc(sizeof(Http2ClientObject), ce);
     zend_object_std_init(&request->std, ce);
     object_properties_init(&request->std, ce);
@@ -311,8 +314,8 @@ void php_swoole_http2_client_coro_minit(int module_number) {
     SW_SET_CLASS_CLONEABLE(swoole_http2_client_coro, sw_zend_class_clone_deny);
     SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http2_client_coro, sw_zend_class_unset_property_deny);
     SW_SET_CLASS_CUSTOM_OBJECT(swoole_http2_client_coro,
-                               php_swoole_http2_client_coro_create_object,
-                               php_swoole_http2_client_coro_free_object,
+                               http2_client_coro_create_object,
+                               http2_client_coro_free_object,
                                Http2ClientObject,
                                std);
 
@@ -339,6 +342,7 @@ void php_swoole_http2_client_coro_minit(int module_number) {
     zend_declare_property_long(swoole_http2_client_coro_ce, ZEND_STRL("sock"), -1, ZEND_ACC_PUBLIC);
     zend_declare_property_long(swoole_http2_client_coro_ce, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
     zend_declare_property_null(swoole_http2_client_coro_ce, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http2_client_coro_ce, ZEND_STRL("socket"), ZEND_ACC_PUBLIC);
     zend_declare_property_bool(swoole_http2_client_coro_ce, ZEND_STRL("connected"), 0, ZEND_ACC_PUBLIC);
     zend_declare_property_null(swoole_http2_client_coro_ce, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
     zend_declare_property_long(swoole_http2_client_coro_ce, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
@@ -387,38 +391,59 @@ void php_swoole_http2_client_coro_minit(int module_number) {
     SW_REGISTER_LONG_CONSTANT("SWOOLE_HTTP2_ERROR_HTTP_1_1_REQUIRED", SW_HTTP2_ERROR_HTTP_1_1_REQUIRED);
 }
 
+void Client::socket_dtor() {
+    socket_ = nullptr;
+    clean_send_queue();
+    auto i = streams.begin();
+    while (i != streams.end()) {
+        destroy_stream(i->second);
+        streams.erase(i++);
+    }
+    if (inflater) {
+        nghttp2_hd_inflate_del(inflater);
+        inflater = nullptr;
+    }
+    if (deflater) {
+        nghttp2_hd_deflate_del(deflater);
+        deflater = nullptr;
+    }
+    zend_update_property_bool(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("connected"), 0);
+    zend_update_property_null(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("socket"));
+    zval_ptr_dtor(&socket_object);
+}
+
 bool Client::connect() {
-    if (sw_unlikely(client != nullptr)) {
+    if (sw_unlikely(socket_ != nullptr)) {
+        update_error_properties(EISCONN, strerror(EISCONN));
         return false;
     }
 
-    client = new Socket(network::Socket::convert_to_type(host));
-    if (UNEXPECTED(client->get_fd() < 0)) {
-        php_swoole_sys_error(E_WARNING, "new Socket() failed");
-        zend_update_property_long(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errCode"), errno);
-        zend_update_property_string(
-            swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errMsg"), swoole_strerror(errno));
-        delete client;
-        client = nullptr;
+    auto object = php_swoole_create_socket(network::Socket::convert_to_type(host));
+    if (UNEXPECTED(!object)) {
+        php_swoole_socket_set_error_properties(zobject, errno, strerror(errno));
         return false;
     }
-    client->set_zero_copy(true);
+
+    ZVAL_OBJ(&socket_object, object);
+    socket_ = php_swoole_get_socket(&socket_object);
+    socket_->set_dtor([this](Socket *_socket) { socket_dtor(); });
+    socket_->set_zero_copy(true);
 #ifdef SW_USE_OPENSSL
     if (open_ssl) {
-        client->enable_ssl_encrypt();
+        socket_->enable_ssl_encrypt();
     }
 #endif
-    client->http2 = 1;
-    client->open_length_check = 1;
-    client->protocol.package_length_size = SW_HTTP2_FRAME_HEADER_SIZE;
-    client->protocol.package_length_offset = 0;
-    client->protocol.package_body_offset = 0;
-    client->protocol.get_package_length = Http2::get_frame_length;
+    socket_->http2 = 1;
+    socket_->open_length_check = 1;
+    socket_->protocol.package_length_size = SW_HTTP2_FRAME_HEADER_SIZE;
+    socket_->protocol.package_length_offset = 0;
+    socket_->protocol.package_body_offset = 0;
+    socket_->protocol.get_package_length = Http2::get_frame_length;
 
     apply_setting(
         sw_zend_read_property_ex(swoole_http2_client_coro_ce, zobject, SW_ZSTR_KNOWN(SW_ZEND_STR_SETTING), 0));
 
-    if (!client->connect(host, port)) {
+    if (!socket_->connect(host, port)) {
         io_error();
         close();
         return false;
@@ -427,6 +452,7 @@ bool Client::connect() {
     stream_id = 1;
     // [init]: we must set default value, server is not always send all the settings
     Http2::init_settings(&remote_settings);
+    remote_window_size = remote_settings.init_window_size;
 
     int ret = nghttp2_hd_inflate_new2(&inflater, php_nghttp2_mem());
     if (ret != 0) {
@@ -451,42 +477,33 @@ bool Client::connect() {
         return false;
     }
 
+    zend_update_property(Z_OBJCE_P(zobject), SW_Z8_OBJ_P(zobject), ZEND_STRL("socket"), &socket_object);
     zend_update_property_bool(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("connected"), 1);
 
     return true;
 }
 
 bool Client::close() {
-    Socket *_client = client;
-    if (!_client) {
+    if (socket_ == nullptr) {
+        update_error_properties(EBADF, strerror(EBADF));
         return false;
     }
-    clean_send_queue();
-    zend_update_property_bool(swoole_http2_client_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("connected"), 0);
-    if (!_client->has_bound()) {
-        auto i = streams.begin();
-        while (i != streams.end()) {
-            destroy_stream(i->second);
-            streams.erase(i++);
-        }
-        if (inflater) {
-            nghttp2_hd_inflate_del(inflater);
-            inflater = nullptr;
-        }
-        if (deflater) {
-            nghttp2_hd_deflate_del(deflater);
-            deflater = nullptr;
-        }
-        client = nullptr;
-    }
-    if (_client->close()) {
-        delete _client;
+    zval tmp_socket = socket_object;
+    zval_add_ref(&tmp_socket);
+    ON_SCOPE_EXIT {
+        zval_ptr_dtor(&tmp_socket);
+    };
+    zend_update_property_bool(Z_OBJCE_P(zobject), SW_Z8_OBJ_P(zobject), ZEND_STRL("connected"), 0);
+    Socket *_socket = php_swoole_get_socket(&tmp_socket);
+    if (!_socket->close()) {
+        update_error_properties(_socket->errCode, _socket->errMsg);
+        return false;
     }
     return true;
 }
 
 ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
-    char *buf = client->get_read_buffer()->str;
+    char *buf = socket_->get_read_buffer()->str;
     uint8_t type = buf[3];
     uint8_t flags = buf[4];
     uint32_t stream_id = ntohl((*(int *) (buf + 5))) & 0x7fffffff;
@@ -530,7 +547,7 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: max_concurrent_streams=%u", value);
                 break;
             case SW_HTTP2_SETTINGS_INIT_WINDOW_SIZE:
-                remote_settings.window_size = value;
+                remote_window_size = remote_settings.init_window_size = value;
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: init_send_window=%u", value);
                 break;
             case SW_HTTP2_SETTINGS_MAX_FRAME_SIZE:
@@ -540,14 +557,13 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
             case SW_HTTP2_SETTINGS_MAX_HEADER_LIST_SIZE:
                 if (value != remote_settings.max_header_list_size) {
                     remote_settings.max_header_list_size = value;
-                    /*
+#if 0
                     int ret = nghttp2_hd_inflate_change_table_size(inflater, value);
-                    if (ret != 0)
-                    {
+                    if (ret != 0) {
                         nghttp2_error(ret, "nghttp2_hd_inflate_change_table_size() failed");
                         return SW_ERROR;
                     }
-                    */
+#endif
                 }
                 swoole_trace_log(SW_TRACE_HTTP2, "setting: max_header_list_size=%u", value);
                 break;
@@ -570,7 +586,7 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
         value = ntohl(*(uint32_t *) buf);
         swoole_http2_frame_trace_log(recv, "window_size_increment=%d", value);
         if (stream_id == 0) {
-            remote_settings.window_size += value;
+            remote_window_size += value;
         } else {
             Stream *stream = get_stream(stream_id);
             if (stream) {
@@ -671,19 +687,19 @@ ReturnCode Client::parse_frame(zval *return_value, bool pipeline_read) {
 
             // now we control the connection flow only (not stream)
             // our window size is unlimited, so we don't worry about subtraction overflow
-            local_settings.window_size -= length;
+            local_window_size -= length;
             stream->local_window_size -= length;
-            if (local_settings.window_size < (SW_HTTP2_MAX_WINDOW_SIZE / 4)) {
-                if (!send_window_update(0, SW_HTTP2_MAX_WINDOW_SIZE - local_settings.window_size)) {
+            if (local_window_size < (local_settings.init_window_size / 4)) {
+                if (!send_window_update(0, local_settings.init_window_size - local_window_size)) {
                     return SW_ERROR;
                 }
-                local_settings.window_size = SW_HTTP2_MAX_WINDOW_SIZE;
+                local_window_size = local_settings.init_window_size;
             }
-            if (stream->local_window_size < (SW_HTTP2_MAX_WINDOW_SIZE / 4)) {
-                if (!send_window_update(stream_id, SW_HTTP2_MAX_WINDOW_SIZE - stream->local_window_size)) {
+            if (stream->local_window_size < (local_settings.init_window_size / 4)) {
+                if (!send_window_update(stream_id, local_settings.init_window_size - stream->local_window_size)) {
                     return SW_ERROR;
                 }
-                stream->local_window_size = SW_HTTP2_MAX_WINDOW_SIZE;
+                stream->local_window_size = local_settings.init_window_size;
             }
         }
     }
@@ -737,10 +753,13 @@ int php_swoole_zlib_decompress(z_stream *stream, String *buffer, char *body, int
     stream->total_in = 0;
     stream->total_out = 0;
 
-#if 0
-    printf(SW_START_LINE"\nstatus=%d\tavail_in=%ld,\tavail_out=%ld,\ttotal_in=%ld,\ttotal_out=%ld\n", status,
-            stream->avail_in, stream->avail_out, stream->total_in, stream->total_out);
-#endif
+    swoole_trace_log(SW_TRACE_ZLIB,
+                     SW_START_LINE "\nstatus=%d\tavail_in=%u,\tavail_out=%u,\ttotal_in=%lu,\ttotal_out=%lu\n",
+                     status,
+                     stream->avail_in,
+                     stream->avail_out,
+                     stream->total_in,
+                     stream->total_out);
 
     buffer->clear();
 
@@ -750,10 +769,15 @@ int php_swoole_zlib_decompress(z_stream *stream, String *buffer, char *body, int
 
         status = inflate(stream, Z_SYNC_FLUSH);
 
-#if 0
-        printf("status=%d\tavail_in=%ld,\tavail_out=%ld,\ttotal_in=%ld,\ttotal_out=%ld,\tlength=%ld\n", status,
-                stream->avail_in, stream->avail_out, stream->total_in, stream->total_out, buffer->length);
-#endif
+        swoole_trace_log(SW_TRACE_ZLIB,
+                         "status=%d\tavail_in=%d,\tavail_out=%d,\ttotal_in=%lu,\ttotal_out=%lu,\tlength=%lu\n",
+                         status,
+                         stream->avail_in,
+                         stream->avail_out,
+                         stream->total_in,
+                         stream->total_out,
+                         buffer->length);
+
         if (status >= 0) {
             buffer->length = stream->total_out;
         }
@@ -794,19 +818,19 @@ static PHP_METHOD(swoole_http2_client_coro, __construct) {
         RETURN_FALSE;
     }
 
-    Client *h2c = new Client(host, host_len, port, ssl, ZEND_THIS);
+    Client *client = new Client(host, host_len, port, ssl, ZEND_THIS);
     if (ssl) {
 #ifndef SW_USE_OPENSSL
         zend_throw_exception_ex(
             swoole_http2_client_coro_exception_ce,
             EPROTONOSUPPORT,
             "you must configure with `--enable-openssl` to support ssl connection when compiling Swoole");
-        delete h2c;
+        delete client;
         RETURN_FALSE;
 #endif
     }
 
-    php_swoole_set_h2c(ZEND_THIS, h2c);
+    http2_client_coro_fetch_object(Z_OBJ_P(ZEND_THIS))->client = client;
 
     zend_update_property_stringl(
         swoole_http2_client_coro_ce, SW_Z8_OBJ_P(ZEND_THIS), ZEND_STRL("host"), host, host_len);
@@ -815,7 +839,7 @@ static PHP_METHOD(swoole_http2_client_coro, __construct) {
 }
 
 static PHP_METHOD(swoole_http2_client_coro, set) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
     zval *zset;
 
     ZEND_PARSE_PARAMETERS_START(1, 1)
@@ -846,48 +870,13 @@ bool Client::send_window_update(int stream_id, uint32_t size) {
  * called on connect
  */
 bool Client::send_setting() {
-    Http2::Settings *settings = &local_settings;
-    uint16_t id = 0;
-    uint32_t value = 0;
-
-    char frame[SW_HTTP2_FRAME_HEADER_SIZE + 18];
-    memset(frame, 0, sizeof(frame));
-    Http2::set_frame_header(frame, SW_HTTP2_TYPE_SETTINGS, 18, 0, 0);
-
-    char *p = frame + SW_HTTP2_FRAME_HEADER_SIZE;
-    /**
-     * HEADER_TABLE_SIZE
-     */
-    id = htons(SW_HTTP2_SETTING_HEADER_TABLE_SIZE);
-    memcpy(p, &id, sizeof(id));
-    p += 2;
-    value = htonl(settings->header_table_size);
-    memcpy(p, &value, sizeof(value));
-    p += 4;
-    /**
-     * MAX_CONCURRENT_STREAMS
-     */
-    id = htons(SW_HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
-    memcpy(p, &id, sizeof(id));
-    p += 2;
-    value = htonl(settings->max_concurrent_streams);
-    memcpy(p, &value, sizeof(value));
-    p += 4;
-    /**
-     * INIT_WINDOW_SIZE
-     */
-    id = htons(SW_HTTP2_SETTINGS_INIT_WINDOW_SIZE);
-    memcpy(p, &id, sizeof(id));
-    p += 2;
-    value = htonl(settings->window_size);
-    memcpy(p, &value, sizeof(value));
-    p += 4;
-
-    swoole_trace_log(SW_TRACE_HTTP2, "[" SW_ECHO_GREEN "]\t[length=%d]", Http2::get_type(SW_HTTP2_TYPE_SETTINGS), 18);
-    return send(frame, SW_HTTP2_FRAME_HEADER_SIZE + 18);
+    char frame[SW_HTTP2_SETTING_FRAME_SIZE];
+    size_t n = Http2::pack_setting_frame(frame, local_settings, false);
+    swoole_trace_log(SW_TRACE_HTTP2, "[" SW_ECHO_GREEN "]\t[length=%lu]", Http2::get_type(SW_HTTP2_TYPE_SETTINGS), n);
+    return send(frame, n);
 }
 
-void http_parse_set_cookies(const char *at, size_t length, zval *zcookies, zval *zset_cookie_headers);
+void php_swoole_http_parse_set_cookies(const char *at, size_t length, zval *zcookies, zval *zset_cookie_headers);
 
 int Client::parse_header(Stream *stream, int flags, char *in, size_t inlen) {
     zval *zresponse = &stream->zresponse;
@@ -942,7 +931,7 @@ int Client::parse_header(Stream *stream, int flags, char *in, size_t inlen) {
             } else {
 #ifdef SW_HAVE_ZLIB
                 if (SW_STRCASEEQ((char *) nv.name, nv.namelen, "content-encoding") &&
-                    SW_STRCASECT((char *) nv.value, nv.valuelen, "gzip")) {
+                    SW_STR_ISTARTS_WITH((char *) nv.value, nv.valuelen, "gzip")) {
                     /**
                      * init zlib stream
                      */
@@ -961,7 +950,7 @@ int Client::parse_header(Stream *stream, int flags, char *in, size_t inlen) {
                 } else
 #endif
                     if (SW_STRCASEEQ((char *) nv.name, nv.namelen, "set-cookie")) {
-                    http_parse_set_cookies((char *) nv.value, nv.valuelen, zcookies, zset_cookie_headers);
+                    php_swoole_http_parse_set_cookies((char *) nv.value, nv.valuelen, zcookies, zset_cookie_headers);
                 }
                 add_assoc_stringl_ex(zheaders, (char *) nv.name, nv.namelen, (char *) nv.value, nv.valuelen);
             }
@@ -978,7 +967,7 @@ int Client::parse_header(Stream *stream, int flags, char *in, size_t inlen) {
 }
 
 ssize_t Client::build_header(zval *zobject, zval *zrequest, char *buffer) {
-    Client *h2c = php_swoole_get_h2c(zobject);
+    Client *h2c = http2_client_coro_get_client(zobject);
     zval *zmethod = sw_zend_read_property_ex(swoole_http2_request_ce, zrequest, SW_ZSTR_KNOWN(SW_ZEND_STR_METHOD), 0);
     zval *zpath = sw_zend_read_property_ex(swoole_http2_request_ce, zrequest, SW_ZSTR_KNOWN(SW_ZEND_STR_PATH), 0);
     zval *zheaders = sw_zend_read_property_ex(swoole_http2_request_ce, zrequest, SW_ZSTR_KNOWN(SW_ZEND_STR_HEADERS), 0);
@@ -1043,7 +1032,7 @@ ssize_t Client::build_header(zval *zobject, zval *zrequest, char *buffer) {
         zend_string *key;
         zval *zvalue;
         char *encoded_value;
-        int encoded_value_len;
+        size_t encoded_value_len;
         String *buffer = sw_tg_buffer();
 
         ZEND_HASH_FOREACH_STR_KEY_VAL(Z_ARRVAL_P(zcookies), key, zvalue) {
@@ -1065,11 +1054,14 @@ ssize_t Client::build_header(zval *zobject, zval *zrequest, char *buffer) {
     }
 
     size_t buflen = nghttp2_hd_deflate_bound(h2c->deflater, headers.get(), headers.len());
-    // if (buflen > h2c->remote_settings.max_header_list_size) {
-    //     php_swoole_error(E_WARNING, "header cannot bigger than remote max_header_list_size %u",
-    //     h2c->remote_settings.max_header_list_size);
-    //     return -1;
-    // }
+#if 0
+    if (buflen > h2c->remote_settings.max_header_list_size) {
+        php_swoole_error(E_WARNING,
+                         "header cannot bigger than remote max_header_list_size %u",
+                         client->remote_settings.max_header_list_size);
+        return -1;
+    }
+#endif
     ssize_t rv = nghttp2_hd_deflate_hd(h2c->deflater, (uchar *) buffer, buflen, headers.get(), headers.len());
     if (rv < 0) {
         h2c->nghttp2_error(rv, "nghttp2_hd_deflate_hd() failed");
@@ -1098,8 +1090,8 @@ Stream *Client::create_stream(uint32_t stream_id, uint8_t flags) {
     // init
     stream->stream_id = stream_id;
     stream->flags = flags;
-    stream->remote_window_size = SW_HTTP2_DEFAULT_WINDOW_SIZE;
-    stream->local_window_size = SW_HTTP2_DEFAULT_WINDOW_SIZE;
+    stream->remote_window_size = remote_settings.init_window_size;
+    stream->local_window_size = local_settings.init_window_size;
     streams.emplace(stream_id, stream);
     // create response object
     object_init_ex(&stream->zresponse, swoole_http2_response_ce);
@@ -1320,7 +1312,7 @@ bool Client::send_goaway_frame(zend_long error_code, const char *debug_data, siz
 }
 
 static PHP_METHOD(swoole_http2_client_coro, send) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
 
     if (!h2c->is_available()) {
         RETURN_FALSE;
@@ -1340,8 +1332,8 @@ static PHP_METHOD(swoole_http2_client_coro, send) {
     }
 }
 
-static void php_swoole_http2_client_coro_recv(INTERNAL_FUNCTION_PARAMETERS, bool pipeline_read) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+static void http2_client_coro_recv(INTERNAL_FUNCTION_PARAMETERS, bool pipeline_read) {
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
 
     double timeout = 0;
 
@@ -1369,32 +1361,32 @@ static void php_swoole_http2_client_coro_recv(INTERNAL_FUNCTION_PARAMETERS, bool
 }
 
 static PHP_METHOD(swoole_http2_client_coro, recv) {
-    php_swoole_http2_client_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
+    http2_client_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
 }
 
 static PHP_METHOD(swoole_http2_client_coro, __destruct) {}
 
 static PHP_METHOD(swoole_http2_client_coro, close) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
     RETURN_BOOL(h2c->close());
 }
 
 static PHP_METHOD(swoole_http2_client_coro, connect) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
     RETURN_BOOL(h2c->connect());
 }
 
-static sw_inline void http2_settings_to_array(Http2::Settings *settings, zval *zarray) {
+static sw_inline void http2_client_settings_to_array(Http2::Settings *settings, zval *zarray) {
     array_init(zarray);
     add_assoc_long_ex(zarray, ZEND_STRL("header_table_size"), settings->header_table_size);
-    add_assoc_long_ex(zarray, ZEND_STRL("window_size"), settings->window_size);
+    add_assoc_long_ex(zarray, ZEND_STRL("init_window_size"), settings->init_window_size);
     add_assoc_long_ex(zarray, ZEND_STRL("max_concurrent_streams"), settings->max_concurrent_streams);
     add_assoc_long_ex(zarray, ZEND_STRL("max_frame_size"), settings->max_frame_size);
     add_assoc_long_ex(zarray, ZEND_STRL("max_header_list_size"), settings->max_header_list_size);
 }
 
 static PHP_METHOD(swoole_http2_client_coro, stats) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
     zval _zarray, *zarray = &_zarray;
     String key = {};
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|s", &key.str, &key.length) == FAILURE) {
@@ -1406,10 +1398,10 @@ static PHP_METHOD(swoole_http2_client_coro, stats) {
         } else if (SW_STREQ(key.str, key.length, "last_stream_id")) {
             RETURN_LONG(h2c->last_stream_id);
         } else if (SW_STREQ(key.str, key.length, "local_settings")) {
-            http2_settings_to_array(&h2c->local_settings, zarray);
+            http2_client_settings_to_array(&h2c->local_settings, zarray);
             RETURN_ZVAL(zarray, 0, 0);
         } else if (SW_STREQ(key.str, key.length, "remote_settings")) {
-            http2_settings_to_array(&h2c->remote_settings, zarray);
+            http2_client_settings_to_array(&h2c->remote_settings, zarray);
             RETURN_ZVAL(zarray, 0, 0);
         } else if (SW_STREQ(key.str, key.length, "active_stream_num")) {
             RETURN_LONG(h2c->streams.size());
@@ -1418,9 +1410,9 @@ static PHP_METHOD(swoole_http2_client_coro, stats) {
         array_init(return_value);
         add_assoc_long_ex(return_value, ZEND_STRL("current_stream_id"), h2c->stream_id);
         add_assoc_long_ex(return_value, ZEND_STRL("last_stream_id"), h2c->last_stream_id);
-        http2_settings_to_array(&h2c->local_settings, zarray);
+        http2_client_settings_to_array(&h2c->local_settings, zarray);
         add_assoc_zval_ex(return_value, ZEND_STRL("local_settings"), zarray);
-        http2_settings_to_array(&h2c->remote_settings, zarray);
+        http2_client_settings_to_array(&h2c->remote_settings, zarray);
         add_assoc_zval_ex(return_value, ZEND_STRL("remote_settings"), zarray);
         add_assoc_long_ex(return_value, ZEND_STRL("active_stream_num"), h2c->streams.size());
     }
@@ -1435,8 +1427,8 @@ static PHP_METHOD(swoole_http2_client_coro, isStreamExist) {
         RETURN_FALSE;
     }
 
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
-    if (!h2c->client) {
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
+    if (!h2c->socket_) {
         RETURN_FALSE;
     } else if (stream_id == 0) {
         RETURN_TRUE;
@@ -1446,7 +1438,7 @@ static PHP_METHOD(swoole_http2_client_coro, isStreamExist) {
 }
 
 static PHP_METHOD(swoole_http2_client_coro, write) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
 
     if (!h2c->is_available()) {
         RETURN_FALSE;
@@ -1462,11 +1454,11 @@ static PHP_METHOD(swoole_http2_client_coro, write) {
 }
 
 static PHP_METHOD(swoole_http2_client_coro, read) {
-    php_swoole_http2_client_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
+    http2_client_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
 }
 
 static PHP_METHOD(swoole_http2_client_coro, ping) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
 
     if (!h2c->is_available()) {
         RETURN_FALSE;
@@ -1485,7 +1477,7 @@ static PHP_METHOD(swoole_http2_client_coro, ping) {
  * +---------------------------------------------------------------+
  */
 static PHP_METHOD(swoole_http2_client_coro, goaway) {
-    Client *h2c = php_swoole_get_h2c(ZEND_THIS);
+    Client *h2c = http2_client_coro_get_client(ZEND_THIS);
     zend_long error_code = SW_HTTP2_ERROR_NO_ERROR;
     char *debug_data = nullptr;
     size_t debug_data_len = 0;
