@@ -31,8 +31,10 @@ BEGIN_EXTERN_C()
 #include "stubs/php_swoole_socket_coro_arginfo.h"
 END_EXTERN_C()
 
+using swoole::HttpProxy;
 using swoole::PacketLength;
 using swoole::Protocol;
+using swoole::Socks5Proxy;
 using swoole::coroutine::Socket;
 using swoole::network::Address;
 
@@ -56,6 +58,7 @@ static PHP_METHOD(swoole_socket_coro, listen);
 static PHP_METHOD(swoole_socket_coro, accept);
 static PHP_METHOD(swoole_socket_coro, connect);
 static PHP_METHOD(swoole_socket_coro, checkLiveness);
+static PHP_METHOD(swoole_socket_coro, getBoundCid);
 static PHP_METHOD(swoole_socket_coro, peek);
 static PHP_METHOD(swoole_socket_coro, recv);
 static PHP_METHOD(swoole_socket_coro, send);
@@ -95,6 +98,7 @@ static const zend_function_entry swoole_socket_coro_methods[] =
     PHP_ME(swoole_socket_coro, accept,         arginfo_class_Swoole_Coroutine_Socket_accept,         ZEND_ACC_PUBLIC)
     PHP_ME(swoole_socket_coro, connect,        arginfo_class_Swoole_Coroutine_Socket_connect,        ZEND_ACC_PUBLIC)
     PHP_ME(swoole_socket_coro, checkLiveness,  arginfo_class_Swoole_Coroutine_Socket_checkLiveness,  ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_socket_coro, getBoundCid,    arginfo_class_Swoole_Coroutine_Socket_getBoundCid,    ZEND_ACC_PUBLIC)
     PHP_ME(swoole_socket_coro, peek,           arginfo_class_Swoole_Coroutine_Socket_peek,           ZEND_ACC_PUBLIC)
     PHP_ME(swoole_socket_coro, recv,           arginfo_class_Swoole_Coroutine_Socket_recv,           ZEND_ACC_PUBLIC)
     PHP_ME(swoole_socket_coro, recvAll,        arginfo_class_Swoole_Coroutine_Socket_recvAll,        ZEND_ACC_PUBLIC)
@@ -127,38 +131,39 @@ static const zend_function_entry swoole_socket_coro_methods[] =
 };
 // clang-format on
 
-#define SW_BAD_SOCKET ((Socket *) -1)
 #define swoole_get_socket_coro(_sock, _zobject)                                                                        \
-    SocketObject *_sock = php_swoole_socket_coro_fetch_object(Z_OBJ_P(_zobject));                                      \
+    SocketObject *_sock = socket_coro_fetch_object(Z_OBJ_P(_zobject));                                                 \
     if (UNEXPECTED(!sock->socket)) {                                                                                   \
         php_swoole_fatal_error(E_ERROR, "you must call Socket constructor first");                                     \
     }                                                                                                                  \
-    if (UNEXPECTED(_sock->socket == SW_BAD_SOCKET)) {                                                                  \
+    if (UNEXPECTED(_sock->socket->is_closed())) {                                                                      \
         zend_update_property_long(swoole_socket_coro_ce, SW_Z8_OBJ_P(_zobject), ZEND_STRL("errCode"), EBADF);          \
         zend_update_property_string(                                                                                   \
             swoole_socket_coro_ce, SW_Z8_OBJ_P(_zobject), ZEND_STRL("errMsg"), strerror(EBADF));                       \
         RETURN_FALSE;                                                                                                  \
     }
 
-static sw_inline SocketObject *php_swoole_socket_coro_fetch_object(zend_object *obj) {
+static sw_inline SocketObject *socket_coro_fetch_object(zend_object *obj) {
     return (SocketObject *) ((char *) obj - swoole_socket_coro_handlers.offset);
 }
 
-static void php_swoole_socket_coro_free_object(zend_object *object) {
-    SocketObject *sock = (SocketObject *) php_swoole_socket_coro_fetch_object(object);
-    if (!sock->reference && sock->socket && sock->socket != SW_BAD_SOCKET) {
+/**
+ * cannot execute close in the destructor, it may be shutting down,
+ * executing close will try to resume other coroutines.
+ */
+static void socket_coro_free_object(zend_object *object) {
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(object);
+    if (!sock->reference && sock->socket) {
         if (!Z_ISUNDEF(sock->zstream)) {
             sock->socket->move_fd();
             zval_ptr_dtor(&sock->zstream);
-        } else {
-            sock->socket->close();
         }
         delete sock->socket;
     }
     zend_object_std_dtor(&sock->std);
 }
 
-static zend_object *php_swoole_socket_coro_create_object(zend_class_entry *ce) {
+static zend_object *socket_coro_create_object(zend_class_entry *ce) {
     SocketObject *sock = (SocketObject *) zend_object_alloc(sizeof(SocketObject), ce);
     zend_object_std_init(&sock->std, ce);
     /* Even if you don't use properties yourself you should still call object_properties_init(),
@@ -169,7 +174,7 @@ static zend_object *php_swoole_socket_coro_create_object(zend_class_entry *ce) {
     return &sock->std;
 }
 
-static void swoole_socket_coro_register_constants(int module_number) {
+static void socket_coro_register_constants(int module_number) {
     REGISTER_LONG_CONSTANT("AF_UNIX", AF_UNIX, CONST_CS | CONST_PERSISTENT);
     REGISTER_LONG_CONSTANT("AF_INET", AF_INET, CONST_CS | CONST_PERSISTENT);
     REGISTER_LONG_CONSTANT("AF_INET6", AF_INET6, CONST_CS | CONST_PERSISTENT);
@@ -710,11 +715,8 @@ void php_swoole_socket_coro_minit(int module_number) {
     SW_SET_CLASS_NOT_SERIALIZABLE(swoole_socket_coro);
     SW_SET_CLASS_CLONEABLE(swoole_socket_coro, sw_zend_class_clone_deny);
     SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_socket_coro, sw_zend_class_unset_property_deny);
-    SW_SET_CLASS_CUSTOM_OBJECT(swoole_socket_coro,
-                               php_swoole_socket_coro_create_object,
-                               php_swoole_socket_coro_free_object,
-                               SocketObject,
-                               std);
+    SW_SET_CLASS_CUSTOM_OBJECT(
+        swoole_socket_coro, socket_coro_create_object, socket_coro_free_object, SocketObject, std);
 
     zend_declare_property_long(swoole_socket_coro_ce, ZEND_STRL("fd"), -1, ZEND_ACC_PUBLIC);
     zend_declare_property_long(swoole_socket_coro_ce, ZEND_STRL("domain"), 0, ZEND_ACC_PUBLIC);
@@ -730,19 +732,19 @@ void php_swoole_socket_coro_minit(int module_number) {
                            swoole_exception);
 
     if (!zend_hash_str_find_ptr(&module_registry, ZEND_STRL("sockets"))) {
-        swoole_socket_coro_register_constants(module_number);
+        socket_coro_register_constants(module_number);
     }
 #ifdef ECANCELED
     SW_REGISTER_LONG_CONSTANT("SOCKET_ECANCELED", ECANCELED);
 #endif
 }
 
-static sw_inline void swoole_socket_coro_sync_properties(zval *zobject, SocketObject *sock) {
+static sw_inline void socket_coro_sync_properties(zval *zobject, SocketObject *sock) {
     zend_update_property_long(swoole_socket_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errCode"), sock->socket->errCode);
     zend_update_property_string(swoole_socket_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errMsg"), sock->socket->errMsg);
 }
 
-static void sw_inline php_swoole_init_socket(zval *zobject, SocketObject *sock) {
+static void sw_inline socket_coro_init(zval *zobject, SocketObject *sock) {
     sock->socket->set_zero_copy(true);
     sock->socket->set_buffer_allocator(sw_zend_string_allocator());
     zend_update_property_long(swoole_socket_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("fd"), sock->socket->get_fd());
@@ -755,17 +757,17 @@ static void sw_inline php_swoole_init_socket(zval *zobject, SocketObject *sock)
 }
 
 SW_API bool php_swoole_export_socket(zval *zobject, Socket *_socket) {
-    zend_object *object = php_swoole_socket_coro_create_object(swoole_socket_coro_ce);
+    zend_object *object = socket_coro_create_object(swoole_socket_coro_ce);
     if (!object) {
         return false;
     }
 
-    SocketObject *sock = (SocketObject *) php_swoole_socket_coro_fetch_object(object);
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(object);
     sock->reference = 1;
     sock->socket = _socket;
 
     ZVAL_OBJ(zobject, object);
-    php_swoole_init_socket(zobject, sock);
+    socket_coro_init(zobject, sock);
     return true;
 }
 
@@ -779,10 +781,38 @@ SW_API zend_object *php_swoole_dup_socket(int fd, enum swSocketType type) {
     return php_swoole_create_socket_from_fd(new_fd, type);
 }
 
+SW_API zend_object *php_swoole_create_socket(enum swSocketType type) {
+    zval zobject;
+    zend_object *object = socket_coro_create_object(swoole_socket_coro_ce);
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(object);
+
+    sock->socket = new Socket(type);
+    if (UNEXPECTED(sock->socket->get_fd() < 0)) {
+        php_swoole_sys_error(E_WARNING, "new Socket() failed");
+        delete sock->socket;
+        sock->socket = nullptr;
+        OBJ_RELEASE(object);
+        return nullptr;
+    }
+
+    ZVAL_OBJ(&zobject, object);
+    socket_coro_init(&zobject, sock);
+    return object;
+}
+
+SW_API void php_swoole_socket_set_error_properties(zval *zobject, int code, const char *msg) {
+    zend_update_property_long(Z_OBJCE_P(zobject), SW_Z8_OBJ_P(zobject), ZEND_STRL("errCode"), code);
+    zend_update_property_string(Z_OBJCE_P(zobject), SW_Z8_OBJ_P(zobject), ZEND_STRL("errMsg"), msg);
+}
+
+SW_API void php_swoole_socket_set_error_properties(zval *zobject, Socket *socket) {
+    php_swoole_socket_set_error_properties(zobject, socket->errCode, socket->errMsg);
+}
+
 SW_API zend_object *php_swoole_create_socket_from_fd(int fd, enum swSocketType type) {
     zval zobject;
-    zend_object *object = php_swoole_socket_coro_create_object(swoole_socket_coro_ce);
-    SocketObject *sock = (SocketObject *) php_swoole_socket_coro_fetch_object(object);
+    zend_object *object = socket_coro_create_object(swoole_socket_coro_ce);
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(object);
 
     sock->socket = new Socket(fd, type);
     if (UNEXPECTED(sock->socket->get_fd() < 0)) {
@@ -794,27 +824,27 @@ SW_API zend_object *php_swoole_create_socket_from_fd(int fd, enum swSocketType t
     }
 
     ZVAL_OBJ(&zobject, object);
-    php_swoole_init_socket(&zobject, sock);
+    socket_coro_init(&zobject, sock);
     return object;
 }
 
 SW_API Socket *php_swoole_get_socket(zval *zobject) {
     SW_ASSERT(Z_OBJCE_P(zobject) == swoole_socket_coro_ce);
-    SocketObject *sock = (SocketObject *) php_swoole_socket_coro_fetch_object(Z_OBJ_P(zobject));
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(Z_OBJ_P(zobject));
     return sock->socket;
 }
 
 SW_API bool php_swoole_socket_is_closed(zval *zobject) {
-    SocketObject *_sock = php_swoole_socket_coro_fetch_object(Z_OBJ_P(zobject));
-    return _sock->socket == SW_BAD_SOCKET || _sock->socket->is_closed();
+    SocketObject *_sock = socket_coro_fetch_object(Z_OBJ_P(zobject));
+    return _sock->socket == nullptr || _sock->socket->is_closed();
 }
 
 SW_API void php_swoole_init_socket_object(zval *zobject, Socket *socket) {
-    zend_object *object = php_swoole_socket_coro_create_object(swoole_socket_coro_ce);
-    SocketObject *sock = (SocketObject *) php_swoole_socket_coro_fetch_object(object);
+    zend_object *object = socket_coro_create_object(swoole_socket_coro_ce);
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(object);
     sock->socket = socket;
     ZVAL_OBJ(zobject, object);
-    php_swoole_init_socket(zobject, sock);
+    socket_coro_init(zobject, sock);
 }
 
 SW_API bool php_swoole_socket_set_protocol(Socket *sock, zval *zset) {
@@ -965,6 +995,215 @@ SW_API bool php_swoole_socket_set_protocol(Socket *sock, zval *zset) {
     return ret;
 }
 
+SW_API bool php_swoole_socket_set(Socket *cli, zval *zset) {
+    HashTable *vht = Z_ARRVAL_P(zset);
+    zval *ztmp;
+    bool ret = true;
+
+    /**
+     * timeout
+     */
+    if (php_swoole_array_get_value(vht, "timeout", ztmp)) {
+        cli->set_timeout(zval_get_double(ztmp));
+    }
+    if (php_swoole_array_get_value(vht, "connect_timeout", ztmp)) {
+        cli->set_timeout(zval_get_double(ztmp), Socket::TIMEOUT_CONNECT);
+    }
+    if (php_swoole_array_get_value(vht, "read_timeout", ztmp)) {
+        cli->set_timeout(zval_get_double(ztmp), Socket::TIMEOUT_READ);
+    }
+    if (php_swoole_array_get_value(vht, "write_timeout", ztmp)) {
+        cli->set_timeout(zval_get_double(ztmp), Socket::TIMEOUT_WRITE);
+    }
+    std::string _bind_address;
+    int _bind_port = 0;
+    if (php_swoole_array_get_value(vht, "bind_port", ztmp)) {
+        zend_long v = zval_get_long(ztmp);
+        _bind_port = SW_MAX(0, SW_MIN(v, UINT16_MAX));
+    }
+    if (php_swoole_array_get_value(vht, "bind_address", ztmp)) {
+        zend::String tmp = ztmp;
+        _bind_address = tmp.to_std_string();
+    }
+    if (!_bind_address.empty() && !cli->bind(_bind_address, _bind_port)) {
+        ret = false;
+    }
+    /**
+     * socket send/recv buffer size
+     */
+    if (php_swoole_array_get_value(vht, "socket_buffer_size", ztmp)) {
+        zend_long size = zval_get_long(ztmp);
+        if (size <= 0) {
+            php_swoole_fatal_error(E_WARNING, "socket buffer size must be greater than 0, got " ZEND_LONG_FMT, size);
+            ret = false;
+        } else {
+            cli->set_option(SOL_SOCKET, SO_RCVBUF, size) && cli->set_option(SOL_SOCKET, SO_SNDBUF, size);
+        }
+    }
+    /**
+     * client: tcp_nodelay
+     */
+    if (php_swoole_array_get_value(vht, "open_tcp_nodelay", ztmp)) {
+        if (cli->get_type() == SW_SOCK_TCP || cli->get_type() != SW_SOCK_TCP6) {
+            cli->get_socket()->set_tcp_nodelay(zval_is_true(ztmp));
+        }
+    }
+    /**
+     * openssl and protocol options
+     */
+    if (!php_swoole_socket_set_protocol(cli, zset)) {
+        ret = false;
+    }
+    /**
+     * socks5 proxy
+     */
+    if (php_swoole_array_get_value(vht, "socks5_host", ztmp)) {
+        zend::String host(ztmp);
+        if (php_swoole_array_get_value(vht, "socks5_port", ztmp)) {
+            if (cli->socks5_proxy == nullptr) {
+                cli->socks5_proxy = new Socks5Proxy();
+            }
+            cli->socks5_proxy->host = host.to_std_string();
+            cli->socks5_proxy->port = zval_get_long(ztmp);
+            cli->socks5_proxy->dns_tunnel = 1;
+            if (php_swoole_array_get_value(vht, "socks5_username", ztmp)) {
+                zend::String username(ztmp);
+                if (username.len() > 0 && php_swoole_array_get_value(vht, "socks5_password", ztmp)) {
+                    zend::String password(ztmp);
+                    if (password.len() > 0) {
+                        cli->socks5_proxy->method = 0x02;
+                        cli->socks5_proxy->username = username.to_std_string();
+                        cli->socks5_proxy->password = password.to_std_string();
+                    }
+                } else {
+                    php_swoole_fatal_error(E_WARNING, "socks5_password should not be null");
+                    ret = false;
+                }
+            }
+        } else {
+            php_swoole_fatal_error(E_WARNING, "socks5_port should not be null");
+            ret = false;
+        }
+    }
+    /**
+     * http proxy
+     */
+    else if (php_swoole_array_get_value(vht, "http_proxy_host", ztmp)) {
+        zend::String host(ztmp);
+        if (php_swoole_array_get_value(vht, "http_proxy_port", ztmp)) {
+            if (cli->http_proxy == nullptr) {
+                cli->http_proxy = new HttpProxy();
+            }
+            cli->http_proxy->proxy_host = host.to_std_string();
+            cli->http_proxy->proxy_port = zval_get_long(ztmp);
+            if (php_swoole_array_get_value(vht, "http_proxy_username", ztmp) ||
+                php_swoole_array_get_value(vht, "http_proxy_user", ztmp)) {
+                zend::String username(ztmp);
+                if (username.len() > 0 && php_swoole_array_get_value(vht, "http_proxy_password", ztmp)) {
+                    zend::String password(ztmp);
+                    if (password.len() > 0) {
+                        cli->http_proxy->username = username.to_std_string();
+                        cli->http_proxy->password = password.to_std_string();
+                    }
+                } else {
+                    php_swoole_fatal_error(E_WARNING, "http_proxy_password should not be null");
+                    ret = false;
+                }
+            }
+        } else {
+            php_swoole_fatal_error(E_WARNING, "http_proxy_port should not be null");
+            ret = false;
+        }
+    }
+
+    return ret;
+}
+
+#ifdef SW_USE_OPENSSL
+SW_API bool php_swoole_socket_set_ssl(Socket *sock, zval *zset) {
+    HashTable *vht = Z_ARRVAL_P(zset);
+    zval *ztmp;
+    bool ret = true;
+
+    if (php_swoole_array_get_value(vht, "ssl_protocols", ztmp)) {
+        zend_long v = zval_get_long(ztmp);
+        sock->get_ssl_context()->protocols = v;
+    }
+    if (php_swoole_array_get_value(vht, "ssl_compress", ztmp)) {
+        sock->get_ssl_context()->disable_compress = !zval_is_true(ztmp);
+    } else if (php_swoole_array_get_value(vht, "ssl_disable_compression", ztmp)) {
+        sock->get_ssl_context()->disable_compress = !zval_is_true(ztmp);
+    }
+    if (php_swoole_array_get_value(vht, "ssl_cert_file", ztmp)) {
+        zend::String str_v(ztmp);
+        if (access(str_v.val(), R_OK) == 0) {
+            sock->get_ssl_context()->cert_file = str_v.to_std_string();
+        } else {
+            php_swoole_fatal_error(E_WARNING, "ssl cert file[%s] not found", str_v.val());
+            ret = false;
+        }
+    }
+    if (php_swoole_array_get_value(vht, "ssl_key_file", ztmp)) {
+        zend::String str_v(ztmp);
+        if (access(str_v.val(), R_OK) == 0) {
+            sock->get_ssl_context()->key_file = str_v.to_std_string();
+        } else {
+            php_swoole_fatal_error(E_WARNING, "ssl key file[%s] not found", str_v.val());
+            ret = false;
+        }
+    }
+    if (!sock->get_ssl_context()->cert_file.empty() && sock->get_ssl_context()->key_file.empty()) {
+        php_swoole_fatal_error(E_WARNING, "ssl require key file");
+    } else if (!sock->get_ssl_context()->key_file.empty() && sock->get_ssl_context()->cert_file.empty()) {
+        php_swoole_fatal_error(E_WARNING, "ssl require cert file");
+    }
+    if (php_swoole_array_get_value(vht, "ssl_passphrase", ztmp)) {
+        sock->get_ssl_context()->passphrase = zend::String(ztmp).to_std_string();
+    }
+#ifdef SSL_CTRL_SET_TLSEXT_HOSTNAME
+    if (php_swoole_array_get_value(vht, "ssl_host_name", ztmp)) {
+        sock->get_ssl_context()->tls_host_name = zend::String(ztmp).to_std_string();
+        /* if user set empty ssl_host_name, disable it, otherwise the underlying may set it automatically */
+        sock->get_ssl_context()->disable_tls_host_name = sock->get_ssl_context()->tls_host_name.empty();
+    }
+#endif
+    if (php_swoole_array_get_value(vht, "ssl_verify_peer", ztmp)) {
+        sock->get_ssl_context()->verify_peer = zval_is_true(ztmp);
+    }
+    if (php_swoole_array_get_value(vht, "ssl_allow_self_signed", ztmp)) {
+        sock->get_ssl_context()->allow_self_signed = zval_is_true(ztmp);
+    }
+    if (php_swoole_array_get_value(vht, "ssl_cafile", ztmp)) {
+        sock->get_ssl_context()->cafile = zend::String(ztmp).to_std_string();
+    }
+    if (php_swoole_array_get_value(vht, "ssl_capath", ztmp)) {
+        sock->get_ssl_context()->capath = zend::String(ztmp).to_std_string();
+    }
+    if (php_swoole_array_get_value(vht, "ssl_verify_depth", ztmp)) {
+        zend_long v = zval_get_long(ztmp);
+        sock->get_ssl_context()->verify_depth = SW_MAX(0, SW_MIN(v, UINT8_MAX));
+    }
+    if (php_swoole_array_get_value(vht, "ssl_ciphers", ztmp)) {
+        sock->get_ssl_context()->ciphers = zend::String(ztmp).to_std_string();
+    }
+    if (php_swoole_array_get_value(vht, "ssl_ecdh_curve", ztmp)) {
+        sock->get_ssl_context()->ecdh_curve = zend::String(ztmp).to_std_string();
+    }
+
+#ifdef OPENSSL_IS_BORINGSSL
+    if (php_swoole_array_get_value(vht, "ssl_grease", ztmp)) {
+        zend_long v = zval_get_long(ztmp);
+        sock->get_ssl_context()->grease = SW_MAX(0, SW_MIN(v, UINT8_MAX));
+    }
+#endif
+
+    if (!sock->ssl_check_context()) {
+        ret = false;
+    }
+    return ret;
+}
+#endif
+
 PHP_FUNCTION(swoole_coroutine_socketpair) {
     zend_long domain, type, protocol;
     php_socket_t pair[2];
@@ -1014,20 +1253,22 @@ static PHP_METHOD(swoole_socket_coro, __construct) {
     Z_PARAM_LONG(protocol)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    SocketObject *sock = (SocketObject *) php_swoole_socket_coro_fetch_object(Z_OBJ_P(ZEND_THIS));
-
-    if (EXPECTED(!sock->socket)) {
-        php_swoole_check_reactor();
-        sock->socket = new Socket((int) domain, (int) type, (int) protocol);
-        if (UNEXPECTED(sock->socket->get_fd() < 0)) {
-            zend_throw_exception_ex(
-                swoole_socket_coro_exception_ce, errno, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
-            delete sock->socket;
-            sock->socket = nullptr;
-            RETURN_FALSE;
-        }
-        php_swoole_init_socket(ZEND_THIS, sock);
+    SocketObject *sock = (SocketObject *) socket_coro_fetch_object(Z_OBJ_P(ZEND_THIS));
+    if (sock->socket) {
+        zend_throw_error(NULL, "Constructor of %s can only be called once", SW_Z_OBJCE_NAME_VAL_P(ZEND_THIS));
+        RETURN_FALSE;
     }
+
+    php_swoole_check_reactor();
+    sock->socket = new Socket((int) domain, (int) type, (int) protocol);
+    if (UNEXPECTED(sock->socket->get_fd() < 0)) {
+        zend_throw_exception_ex(
+            swoole_socket_coro_exception_ce, errno, "new Socket() failed. Error: %s [%d]", strerror(errno), errno);
+        delete sock->socket;
+        sock->socket = nullptr;
+        RETURN_FALSE;
+    }
+    socket_coro_init(ZEND_THIS, sock);
 }
 
 static PHP_METHOD(swoole_socket_coro, bind) {
@@ -1044,7 +1285,7 @@ static PHP_METHOD(swoole_socket_coro, bind) {
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     if (!sock->socket->bind(std::string(address, l_address), port)) {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -1061,7 +1302,7 @@ static PHP_METHOD(swoole_socket_coro, listen) {
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     if (!sock->socket->listen(backlog)) {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -1079,8 +1320,8 @@ static PHP_METHOD(swoole_socket_coro, accept) {
 
     Socket *conn = sock->socket->accept(timeout);
     if (conn) {
-        zend_object *client = php_swoole_socket_coro_create_object(swoole_socket_coro_ce);
-        SocketObject *client_sock = (SocketObject *) php_swoole_socket_coro_fetch_object(client);
+        zend_object *client = socket_coro_create_object(swoole_socket_coro_ce);
+        SocketObject *client_sock = (SocketObject *) socket_coro_fetch_object(client);
         client_sock->socket = conn;
         ZVAL_OBJ(return_value, &client_sock->std);
         if (conn->protocol.private_data) {
@@ -1089,9 +1330,9 @@ static PHP_METHOD(swoole_socket_coro, accept) {
             sw_zend_fci_cache_persist(fci_cache);
             conn->protocol.private_data = fci_cache;
         }
-        php_swoole_init_socket(return_value, client_sock);
+        socket_coro_init(return_value, client_sock);
     } else {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
 }
@@ -1122,7 +1363,7 @@ static PHP_METHOD(swoole_socket_coro, connect) {
     }
     Socket::TimeoutSetter ts(sock->socket, timeout, Socket::TIMEOUT_CONNECT);
     if (!sock->socket->connect(std::string(host, l_host), port)) {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -1132,10 +1373,20 @@ static PHP_METHOD(swoole_socket_coro, checkLiveness) {
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     bool liveness = sock->socket->check_liveness();
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     RETURN_BOOL(liveness);
 }
 
+static PHP_METHOD(swoole_socket_coro, getBoundCid) {
+    zend_long event;
+    ZEND_PARSE_PARAMETERS_START(1, 1)
+    Z_PARAM_LONG(event)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    swoole_get_socket_coro(sock, ZEND_THIS);
+    RETURN_LONG(sock->socket->get_bound_cid((swEventType) event));
+}
+
 static PHP_METHOD(swoole_socket_coro, peek) {
     zend_long length = SW_BUFFER_SIZE_BIG;
 
@@ -1152,7 +1403,7 @@ static PHP_METHOD(swoole_socket_coro, peek) {
 
     zend_string *buf = zend_string_alloc(length, 0);
     ssize_t bytes = sock->socket->peek(ZSTR_VAL(buf), length);
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     if (UNEXPECTED(bytes < 0)) {
         zend_string_free(buf);
         RETURN_FALSE;
@@ -1171,7 +1422,7 @@ enum RecvMode {
     SOCKET_RECV_WITH_BUFFER,
 };
 
-static inline void swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAMETERS, RecvMode type) {
+static inline void socket_coro_recv(INTERNAL_FUNCTION_PARAMETERS, RecvMode type) {
     zend_long length = SW_BUFFER_SIZE_BIG;
     double timeout = 0;
 
@@ -1207,7 +1458,7 @@ static inline void swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAMETERS, RecvMod
         assert(0);
         break;
     }
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     if (UNEXPECTED(bytes < 0)) {
         zend_string_free(buf);
         RETURN_FALSE;
@@ -1220,19 +1471,19 @@ static inline void swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAMETERS, RecvMod
 }
 
 static PHP_METHOD(swoole_socket_coro, recv) {
-    swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV);
+    socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV);
 }
 
 static PHP_METHOD(swoole_socket_coro, recvAll) {
-    swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV_ALL);
+    socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV_ALL);
 }
 
 static PHP_METHOD(swoole_socket_coro, recvLine) {
-    swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV_LINE);
+    socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV_LINE);
 }
 
 static PHP_METHOD(swoole_socket_coro, recvWithBuffer) {
-    swoole_socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV_WITH_BUFFER);
+    socket_coro_recv(INTERNAL_FUNCTION_PARAM_PASSTHRU, SOCKET_RECV_WITH_BUFFER);
 }
 
 static PHP_METHOD(swoole_socket_coro, recvPacket) {
@@ -1245,7 +1496,7 @@ static PHP_METHOD(swoole_socket_coro, recvPacket) {
 
     swoole_get_socket_coro(sock, ZEND_THIS);
     ssize_t retval = sock->socket->recv_packet(timeout);
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     if (retval < 0) {
         RETURN_FALSE;
     } else if (retval == 0) {
@@ -1262,7 +1513,7 @@ static PHP_METHOD(swoole_socket_coro, recvPacket) {
     }
 }
 
-static sw_inline void swoole_socket_coro_send(INTERNAL_FUNCTION_PARAMETERS, const bool all) {
+static sw_inline void socket_coro_send(INTERNAL_FUNCTION_PARAMETERS, const bool all) {
     char *data;
     size_t length;
     double timeout = 0;
@@ -1277,7 +1528,7 @@ static sw_inline void swoole_socket_coro_send(INTERNAL_FUNCTION_PARAMETERS, cons
 
     Socket::TimeoutSetter ts(sock->socket, timeout, Socket::TIMEOUT_WRITE);
     ssize_t retval = all ? sock->socket->send_all(data, length) : sock->socket->send(data, length);
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     if (UNEXPECTED(retval < 0)) {
         RETURN_FALSE;
     } else {
@@ -1286,10 +1537,10 @@ static sw_inline void swoole_socket_coro_send(INTERNAL_FUNCTION_PARAMETERS, cons
 }
 
 static PHP_METHOD(swoole_socket_coro, send) {
-    swoole_socket_coro_send(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
+    socket_coro_send(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
 }
 
-static void swoole_socket_coro_write_vector(INTERNAL_FUNCTION_PARAMETERS, const bool all) {
+static void socket_coro_write_vector(INTERNAL_FUNCTION_PARAMETERS, const bool all) {
     zval *ziov = nullptr;
     zval *zelement = nullptr;
     HashTable *vht;
@@ -1306,7 +1557,7 @@ static void swoole_socket_coro_write_vector(INTERNAL_FUNCTION_PARAMETERS, const
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     ON_SCOPE_EXIT {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
     };
 
     vht = Z_ARRVAL_P(ziov);
@@ -1351,14 +1602,14 @@ static void swoole_socket_coro_write_vector(INTERNAL_FUNCTION_PARAMETERS, const
 }
 
 static PHP_METHOD(swoole_socket_coro, writeVector) {
-    swoole_socket_coro_write_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
+    socket_coro_write_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
 }
 
 static PHP_METHOD(swoole_socket_coro, writeVectorAll) {
-    swoole_socket_coro_write_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
+    socket_coro_write_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
 }
 
-static void swoole_socket_coro_read_vector(INTERNAL_FUNCTION_PARAMETERS, const bool all) {
+static void socket_coro_read_vector(INTERNAL_FUNCTION_PARAMETERS, const bool all) {
     zval *ziov = nullptr;
     zval *zelement = nullptr;
     HashTable *vht;
@@ -1376,7 +1627,7 @@ static void swoole_socket_coro_read_vector(INTERNAL_FUNCTION_PARAMETERS, const b
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     ON_SCOPE_EXIT {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
     };
 
     vht = Z_ARRVAL_P(ziov);
@@ -1459,11 +1710,11 @@ static void swoole_socket_coro_read_vector(INTERNAL_FUNCTION_PARAMETERS, const b
 }
 
 static PHP_METHOD(swoole_socket_coro, readVector) {
-    swoole_socket_coro_read_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
+    socket_coro_read_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
 }
 
 static PHP_METHOD(swoole_socket_coro, readVectorAll) {
-    swoole_socket_coro_read_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
+    socket_coro_read_vector(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
 }
 
 static PHP_METHOD(swoole_socket_coro, sendFile) {
@@ -1493,7 +1744,7 @@ static PHP_METHOD(swoole_socket_coro, sendFile) {
 }
 
 static PHP_METHOD(swoole_socket_coro, sendAll) {
-    swoole_socket_coro_send(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
+    socket_coro_send(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
 }
 
 static PHP_METHOD(swoole_socket_coro, recvfrom) {
@@ -1511,7 +1762,7 @@ static PHP_METHOD(swoole_socket_coro, recvfrom) {
     zend_string *buf = zend_string_alloc(SW_BUFFER_SIZE_BIG, 0);
     Socket::TimeoutSetter ts(sock->socket, timeout, Socket::TIMEOUT_READ);
     ssize_t bytes = sock->socket->recvfrom(ZSTR_VAL(buf), SW_BUFFER_SIZE_BIG);
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     if (bytes < 0) {
         zend_string_free(buf);
         RETURN_FALSE;
@@ -1546,7 +1797,7 @@ static PHP_METHOD(swoole_socket_coro, sendto) {
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     ssize_t retval = sock->socket->sendto(std::string(addr, l_addr), port, data, l_data);
-    swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+    socket_coro_sync_properties(ZEND_THIS, sock);
     if (retval < 0) {
         RETURN_FALSE;
     } else {
@@ -1565,7 +1816,7 @@ static PHP_METHOD(swoole_socket_coro, shutdown) {
     swoole_get_socket_coro(sock, ZEND_THIS);
 
     if (!sock->socket->shutdown(how)) {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -1594,13 +1845,8 @@ static PHP_METHOD(swoole_socket_coro, close) {
         }
         ZVAL_UNDEF(&sock->zstream);
         sock->socket->move_fd();
-        delete sock->socket;
-        sock->socket = SW_BAD_SOCKET;
     } else {
-        if (sock->socket->close()) {
-            delete sock->socket;
-            sock->socket = SW_BAD_SOCKET;
-        }
+        sock->socket->close();
     }
     RETURN_TRUE;
 }
@@ -1610,7 +1856,7 @@ static PHP_METHOD(swoole_socket_coro, getsockname) {
 
     Address sa;
     if (!sock->socket->getsockname(&sa)) {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
 
@@ -1624,7 +1870,7 @@ static PHP_METHOD(swoole_socket_coro, getpeername) {
 
     Address sa;
     if (!sock->socket->getpeername(&sa)) {
-        swoole_socket_coro_sync_properties(ZEND_THIS, sock);
+        socket_coro_sync_properties(ZEND_THIS, sock);
         RETURN_FALSE;
     }
 
@@ -1937,7 +2183,7 @@ static PHP_METHOD(swoole_socket_coro, import) {
         RETURN_FALSE;
     }
     zend_object *object = php_swoole_create_socket_from_fd(socket_fd, type);
-    SocketObject *sock = php_swoole_socket_coro_fetch_object(object);
+    SocketObject *sock = socket_coro_fetch_object(object);
 
     ZVAL_COPY(&sock->zstream, zstream);
     php_stream_set_option(stream, PHP_STREAM_OPTION_READ_BUFFER, PHP_STREAM_BUFFER_NONE, NULL);
