@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   |         Twosee  <twose@qq.com>                                       |
   +----------------------------------------------------------------------+
 */
@@ -122,6 +122,7 @@ struct Worker {
     uint8_t child_process;
 
     sw_atomic_t tasking_num;
+    uint32_t concurrency;
     time_t start_time;
 
     sw_atomic_long_t dispatch_count;
@@ -279,7 +280,7 @@ struct ProcessPool {
     int response(const char *data, int length);
     int dispatch_blocking(EventData *data, int *dst_worker_id);
     int dispatch_blocking(const char *data, uint32_t len);
-    int add_worker(Worker *worker);
+    void add_worker(Worker *worker);
     int del_worker(Worker *worker);
     void destroy();
     int create(uint32_t worker_num, key_t msgqueue_key = 0, swIPCMode ipc_mode = SW_IPC_NONE);
