@@ -63,11 +63,10 @@ void *Channel::pop(double timeout) {
         msg.error = false;
         msg.timer = nullptr;
         if (timeout > 0) {
-            long msec = (long) (timeout * 1000);
             msg.chan = this;
             msg.type = CONSUMER;
             msg.co = current_co;
-            msg.timer = swoole_timer_add(msec, false, timer_callback, &msg);
+            msg.timer = swoole_timer_add(timeout, false, timer_callback, &msg);
         }
 
         yield(CONSUMER);
@@ -114,11 +113,10 @@ bool Channel::push(void *data, double timeout) {
         msg.error = false;
         msg.timer = nullptr;
         if (timeout > 0) {
-            long msec = (long) (timeout * 1000);
             msg.chan = this;
             msg.type = PRODUCER;
             msg.co = current_co;
-            msg.timer = swoole_timer_add(msec, false, timer_callback, &msg);
+            msg.timer = swoole_timer_add(timeout, false, timer_callback, &msg);
         }
 
         yield(PRODUCER);
