@@ -31,8 +31,8 @@
 namespace swoole {
 namespace coroutine {
 
-Context::Context(size_t stack_size, const CoroutineFunc &fn, void *private_data)
-    : fn_(fn), stack_size_(stack_size), private_data_(private_data) {
+Context::Context(size_t stack_size, CoroutineFunc fn, void *private_data)
+    : fn_(std::move(fn)), stack_size_(stack_size), private_data_(private_data) {
     end_ = false;
 
 #ifdef SW_CONTEXT_PROTECT_STACK_PAGE
@@ -138,7 +138,7 @@ bool Context::swap_out() {
 }
 
 void Context::context_func(void *arg) {
-    Context *_this = (Context *) arg;
+    auto *_this = (Context *) arg;
     _this->fn_(_this->private_data_);
     _this->end_ = true;
     _this->swap_out();
