@@ -30,18 +30,21 @@
 using swoole::AsyncEvent;
 using swoole::Coroutine;
 using swoole::async::dispatch;
+using swoole::coroutine::async;
+using swoole::coroutine::PollSocket;
 using swoole::coroutine::Socket;
 using swoole::coroutine::System;
-using swoole::coroutine::async;
+using swoole::coroutine::translate_events_to_poll;
+using swoole::coroutine::translate_events_from_poll;
 
-static std::unordered_map<int, Socket *> socket_map;
+static std::unordered_map<int, std::shared_ptr<Socket>> socket_map;
 static std::mutex socket_map_lock;
 
 static sw_inline bool is_no_coro() {
     return SwooleTG.reactor == nullptr || !Coroutine::get_current();
 }
 
-static sw_inline Socket *get_socket(int sockfd) {
+static sw_inline std::shared_ptr<Socket> get_socket(int sockfd) {
     std::unique_lock<std::mutex> _lock(socket_map_lock);
     auto socket_iterator = socket_map.find(sockfd);
     if (socket_iterator == socket_map.end()) {
@@ -50,14 +53,14 @@ static sw_inline Socket *get_socket(int sockfd) {
     return socket_iterator->second;
 }
 
-static sw_inline Socket *get_socket_ex(int sockfd) {
+static sw_inline std::shared_ptr<Socket> get_socket_ex(int sockfd) {
     if (sw_unlikely(is_no_coro())) {
         return nullptr;
     }
     return get_socket(sockfd);
 }
 
-Socket *swoole_coroutine_get_socket_object(int sockfd) {
+std::shared_ptr<Socket> swoole_coroutine_get_socket_object(int sockfd) {
     return get_socket(sockfd);
 }
 
@@ -67,10 +70,10 @@ int swoole_coroutine_socket(int domain, int type, int protocol) {
     if (sw_unlikely(is_no_coro())) {
         return ::socket(domain, type, protocol);
     }
-    Socket *socket = new Socket(domain, type, protocol);
+    auto socket = std::make_shared<Socket>(domain, type, protocol);
     int fd = socket->get_fd();
     if (sw_unlikely(fd < 0)) {
-        delete socket;
+        return -1;
     } else {
         std::unique_lock<std::mutex> _lock(socket_map_lock);
         socket_map[fd] = socket;
@@ -79,32 +82,32 @@ int swoole_coroutine_socket(int domain, int type, int protocol) {
 }
 
 ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         return ::send(sockfd, buf, len, flags);
     }
     return socket->send(buf, len);
 }
 
 ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         return ::sendmsg(sockfd, msg, flags);
     }
     return socket->sendmsg(msg, flags);
 }
 
 ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         return ::recvmsg(sockfd, msg, flags);
     }
     return socket->recvmsg(msg, flags);
 }
 
 ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         return ::recv(sockfd, buf, len, flags);
     }
     if (flags & MSG_PEEK) {
@@ -115,30 +118,34 @@ ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags) {
 }
 
 int swoole_coroutine_close(int sockfd) {
-    Socket *socket = get_socket(sockfd);
-    if (socket == NULL) {
+    auto socket = get_socket(sockfd);
+    if (socket == nullptr) {
         return ::close(sockfd);
     }
     if (socket->close()) {
-        delete socket;
         std::unique_lock<std::mutex> _lock(socket_map_lock);
         socket_map.erase(sockfd);
+        return 0;
     }
-    return 0;
+    return -1;
 }
 
 int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         return ::connect(sockfd, addr, addrlen);
     }
     return socket->connect(addr, addrlen) ? 0 : -1;
 }
 
-#if 1
-int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
-    Socket *socket;
-    if (sw_unlikely(nfds != 1 || timeout == 0 || (socket = get_socket_ex(fds[0].fd)) == NULL)) {
+int swoole_coroutine_poll_fake(struct pollfd *fds, nfds_t nfds, int timeout) {
+    if (nfds != 1) {
+        swoole_set_last_error(SW_ERROR_INVALID_PARAMS);
+        swoole_warning("fake poll() implementation, only supports one socket");
+        return -1;
+    }
+    auto socket = get_socket_ex(fds[0].fd);
+    if (sw_unlikely(timeout == 0 || socket == nullptr)) {
         return poll(fds, nfds, timeout);
     }
     socket->set_timeout((double) timeout / 1000);
@@ -150,26 +157,26 @@ int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
     }
     return 1;
 }
-#else
+
 int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
-    if (sw_unlikely(is_no_coro() || nfds != 1 || timeout == 0)) {
+    if (sw_unlikely(is_no_coro() || timeout == 0)) {
         return poll(fds, nfds, timeout);
     }
 
-    std::unordered_map<int, swoole::socket_poll_fd> _fds;
-    for (int i = 0; i < nfds; i++) {
-        _fds.emplace(std::make_pair(fds[i].fd, swoole::socket_poll_fd(fds[i].events, &fds[i])));
+    std::unordered_map<int, PollSocket> _fds;
+    for (nfds_t i = 0; i < nfds; i++) {
+        _fds.emplace(std::make_pair(fds[i].fd, PollSocket(translate_events_from_poll(fds[i].events), &fds[i])));
     }
 
     if (!System::socket_poll(_fds, (double) timeout / 1000)) {
         return -1;
     }
 
-    int retval;
+    int retval = 0;
     for (auto &i : _fds) {
         int revents = i.second.revents;
         struct pollfd *_fd = (struct pollfd *) i.second.ptr;
-        _fd->revents = revents;
+        _fd->revents = translate_events_to_poll(revents);
         if (revents > 0) {
             retval++;
         }
@@ -177,7 +184,6 @@ int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
 
     return retval;
 }
-#endif
 
 int swoole_coroutine_open(const char *pathname, int flags, mode_t mode) {
     if (sw_unlikely(is_no_coro())) {
@@ -193,15 +199,15 @@ int swoole_coroutine_socket_create(int fd) {
     if (sw_unlikely(is_no_coro())) {
         return -1;
     }
-    Socket *socket = new Socket(fd, SW_SOCK_RAW);
+    auto socket = std::make_shared<Socket>(fd, SW_SOCK_RAW);
     int _fd = socket->get_fd();
     if (sw_unlikely(_fd < 0)) {
-        delete socket;
+        return -1;
     } else {
         std::unique_lock<std::mutex> _lock(socket_map_lock);
         socket_map[fd] = socket;
+        return 0;
     }
-    return 0;
 }
 
 uint8_t swoole_coroutine_socket_exists(int fd) {
@@ -213,8 +219,8 @@ ssize_t swoole_coroutine_read(int sockfd, void *buf, size_t count) {
         return read(sockfd, buf, count);
     }
 
-    Socket *socket = get_socket(sockfd);
-    if (socket) {
+    auto socket = get_socket(sockfd);
+    if (socket != nullptr) {
         return socket->read(buf, count);
     }
 
@@ -228,8 +234,8 @@ ssize_t swoole_coroutine_write(int sockfd, const void *buf, size_t count) {
         return write(sockfd, buf, count);
     }
 
-    Socket *socket = get_socket(sockfd);
-    if (socket) {
+    auto socket = get_socket(sockfd);
+    if (socket != nullptr) {
         return socket->write(buf, count);
     }
 
@@ -445,9 +451,7 @@ struct dirent *swoole_coroutine_readdir(DIR *dirp) {
 
     struct dirent *retval;
 
-    async([&retval, dirp]() {
-        retval = readdir(dirp);
-    });
+    async([&retval, dirp]() { retval = readdir(dirp); });
 
     return retval;
 }
@@ -471,8 +475,8 @@ void swoole_coroutine_usleep(int usec) {
 }
 
 int swoole_coroutine_socket_set_timeout(int sockfd, int which, double timeout) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         errno = EINVAL;
         return -1;
     }
@@ -489,8 +493,8 @@ int swoole_coroutine_socket_set_timeout(int sockfd, int which, double timeout) {
 }
 
 int swoole_coroutine_socket_set_connect_timeout(int sockfd, double timeout) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         errno = EINVAL;
         return -1;
     }
@@ -499,8 +503,8 @@ int swoole_coroutine_socket_set_connect_timeout(int sockfd, double timeout) {
 }
 
 int swoole_coroutine_socket_wait_event(int sockfd, int event, double timeout) {
-    Socket *socket = get_socket_ex(sockfd);
-    if (sw_unlikely(socket == NULL)) {
+    auto socket = get_socket_ex(sockfd);
+    if (sw_unlikely(socket == nullptr)) {
         errno = EINVAL;
         return -1;
     }
@@ -515,22 +519,14 @@ int swoole_coroutine_getaddrinfo(const char *name,
                                  const char *service,
                                  const struct addrinfo *req,
                                  struct addrinfo **pai) {
-    if (sw_unlikely(is_no_coro())) {
-        return getaddrinfo(name, service, req, pai);
-    }
-
     int retval = -1;
     async([&]() { retval = getaddrinfo(name, service, req, pai); });
     return retval;
 }
 
 struct hostent *swoole_coroutine_gethostbyname(const char *name) {
-    if (sw_unlikely(is_no_coro())) {
-        return gethostbyname(name);
-    }
-
     struct hostent *retval = nullptr;
-    int _tmp_h_errno;
+    int _tmp_h_errno = 0;
     async([&]() {
         retval = gethostbyname(name);
         _tmp_h_errno = h_errno;
