@@ -60,8 +60,8 @@ void thread_context_clean() {
     g_lock.unlock();
 }
 
-Context::Context(size_t stack_size, const CoroutineFunc &fn, void *private_data)
-    : fn_(fn), private_data_(private_data) {
+Context::Context(size_t stack_size, CoroutineFunc fn, void *private_data)
+    : fn_(std::move(fn)), private_data_(private_data) {
     end_ = false;
     lock_.lock();
     swap_lock_ = nullptr;
