@@ -78,7 +78,7 @@ int ProcessPool::create(uint32_t _worker_num, key_t _msgqueue_key, swIPCMode _ip
         return SW_ERR;
     }
 
-    if (create_message_box(65536) < 0) {
+    if (create_message_box(SW_MESSAGE_BOX_SIZE) < 0) {
         return SW_ERR;
     }
 
@@ -181,6 +181,7 @@ int ProcessPool::start() {
     uint32_t i;
     running = started = true;
     master_pid = getpid();
+    reload_workers = new Worker[worker_num]();
 
     for (i = 0; i < worker_num; i++) {
         workers[i].pool = this;
@@ -388,21 +389,13 @@ pid_t ProcessPool::spawn(Worker *worker) {
     switch (pid) {
     // child
     case 0:
-        /**
-         * Process start
-         */
+        worker->pid = SwooleG.pid;
         if (onWorkerStart != nullptr) {
             onWorkerStart(this, worker->id);
         }
-        /**
-         * Process main loop
-         */
         if (main_loop) {
             ret_code = main_loop(this, worker);
         }
-        /**
-         * Process stop
-         */
         if (onWorkerStop != nullptr) {
             onWorkerStop(this, worker->id);
         }
@@ -650,12 +643,8 @@ static int ProcessPool_worker_loop_ex(ProcessPool *pool, Worker *worker) {
     return SW_OK;
 }
 
-/**
- * add a worker to pool
- */
-int ProcessPool_add_worker(ProcessPool *pool, Worker *worker) {
-    pool->map_->emplace(std::make_pair(worker->pid, worker));
-    return SW_OK;
+void ProcessPool::add_worker(Worker *worker) {
+    map_->emplace(std::make_pair(worker->pid, worker));
 }
 
 bool ProcessPool::detach() {
@@ -679,12 +668,6 @@ int ProcessPool::wait() {
     pid_t new_pid, reload_worker_pid = 0;
     int ret;
 
-    reload_workers = new Worker[worker_num]();
-    ON_SCOPE_EXIT {
-        delete[] reload_workers;
-        reload_workers = nullptr;
-    };
-
     while (running) {
         ExitStatus exit_status = wait_process();
 
@@ -728,7 +711,6 @@ int ProcessPool::wait() {
                 continue;
             } else {
                 if (!reload_init) {
-                    swoole_info("reload workers");
                     reload_init = true;
                     memcpy(reload_workers, workers, sizeof(Worker) * worker_num);
                     if (max_wait_time) {
@@ -830,6 +812,10 @@ void ProcessPool::destroy() {
         message_box->destroy();
     }
 
+    if (reload_workers) {
+        delete[] reload_workers;
+    }
+
     sw_mem_pool()->free(workers);
 }
 
