@@ -70,11 +70,6 @@ static void signal_init() {
     if (!signal_ready) {
         Reactor *reactor = SwooleTG.reactor;
         swoole_signal_set(SIGCHLD, signal_handler);
-#ifdef HAVE_SIGNALFD
-        if (SwooleG.use_signalfd && !reactor->isset_handler(SW_FD_SIGNAL)) {
-            swoole_signalfd_setup(reactor);
-        }
-#endif
 
         reactor->set_exit_condition(Reactor::EXIT_CONDITION_WAIT_PID, [](Reactor *reactor, size_t &event_num) -> bool {
             return swoole_coroutine_wait_count() == 0;
