@@ -687,12 +687,6 @@ int Server::start_reactor_threads() {
 
     Reactor *reactor = sw_reactor();
 
-#ifdef HAVE_SIGNALFD
-    if (SwooleG.use_signalfd) {
-        swoole_signalfd_setup(reactor);
-    }
-#endif
-
     for (auto iter = ports.begin(); iter != ports.end(); iter++) {
         auto port = *iter;
         if (port->is_dgram()) {
@@ -947,7 +941,7 @@ int Server::dispatch_task(const Protocol *proto, Socket *_socket, const RecvData
             return_code = SW_ERR;
             goto _return;
         }
-        if (rdata->data && rdata->info.len >= 0 && stream->send(rdata->data, rdata->info.len) < 0) {
+        if (rdata->data && rdata->info.len > 0 && stream->send(rdata->data, rdata->info.len) < 0) {
             goto _cancel;
         }
     } else {
