@@ -2,17 +2,17 @@
 
 $start = microtime(true);
 
-$cli = new \swoole_client(SWOOLE_SOCK_TCP, SWOOLE_SOCK_ASYNC);
-$cli->on("connect", function(swoole_client $cli) {
-    assert(false);
+$cli = new Swoole\Client(SWOOLE_SOCK_TCP, SWOOLE_SOCK_ASYNC);
+$cli->on("connect", function(Swoole\Client $cli) {
+    Assert::true(false, 'never here');
 });
-$cli->on("receive", function(swoole_client $cli, $data) {
-    assert(false);
+$cli->on("receive", function(Swoole\Client $cli, $data) {
+    Assert::true(false, 'never here');
 });
-$cli->on("error", function(swoole_client $cli) {
+$cli->on("error", function(Swoole\Client $cli) {
     echo "error\n";
 });
-$cli->on("close", function(swoole_client $cli) {
+$cli->on("close", function(Swoole\Client $cli) {
     echo "close\n";
 });
 
@@ -31,4 +31,4 @@ function refcount($var)
 @$cli->connect("11.11.11.11", 9000, 0.1);
 Swoole\Event::wait();
 // xdebug_debug_zval("cli");
-// echo refcount($cli); // php7无效
\ No newline at end of file
+// echo refcount($cli); // php7无效
